/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.news;

import javax.annotation.Nonnull;
import javax.inject.Provider;
import java.io.IOException;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.semantic.document.Document;
import it.tidalwave.mobile.rss.RSSFeed;
import it.tidalwave.mobile.ui.QuestionWithFeedback;
import lombok.RequiredArgsConstructor;
import static org.openide.util.NbBundle.*;

/***********************************************************************************************************************
 *
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor
public class DefaultNewsViewController implements NewsViewController
  {
    private static final Class<DefaultNewsService> _ = DefaultNewsService.class;
    
    @Nonnull
    private final NewsView view;
    
    private final Provider<NewsService> newsService = Locator.createProviderFor(NewsService.class);
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public void loadNewsFeed()
      {
//        logger.info("loadNewsFeed()");

        newsService.get().getNewsFeed(new NewsService.NewsFeedCallback() 
          {
            public void notifyFeedAvailable (final @Nonnull RSSFeed newsFeed) 
              {
                view.notifyFeedAvailable(newsFeed);
              }

            public void notifyFeedUnavailable() 
              {
                view.notifyFeedUnavailable();
              }

            public void notifyFeedCouldBeDownloaded() 
              {
                final String title = getMessage(_, "confirmDownloadTitle");
                final String message = getMessage(_, "confirmDownloadMessage");
                final NewsService.NewsFeedCallback this_ = this;

                view.confirmToDownloadNews(new QuestionWithFeedback(title, message)
                  {
                    @Override
                    public void onConfirm()
                      {
                        newsService.get().downloadNewsFeedInBackground(this_);
                      }

                    @Override
                    public void onCancel()
                      {
                        view.dismiss();
                      }
                  });
              }
          });
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override 
    public void markRead (final @Nonnull Document message)
      throws IOException
      {
        newsService.get().markRead(message);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override 
    public boolean isRead (final @Nonnull Document message)
      {
        return newsService.get().isRead(message);
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void markAllRead (final @Nonnull RSSFeed rssFeed)
      throws IOException
      {
        newsService.get().markAllRead(rssFeed);
        view.notifyAllMessagesMarkedAsRead(getMessage(_, "allMarkedAsRead"));
      }
  }
