/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.news;

import javax.annotation.Nonnull;
import it.tidalwave.mobile.ui.QuestionWithFeedback;
import it.tidalwave.mobile.rss.RSSFeed;
import it.tidalwave.util.thread.ThreadType;
import it.tidalwave.util.thread.annotation.ThreadConfined;

/***********************************************************************************************************************
 *
 * @stereotype View
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface NewsView
  {
    /*******************************************************************************************************************
     *
     * Dismisses this UI.
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void dismiss();

    /*******************************************************************************************************************
     *
     * Notifies that there's a news feed available.
     * 
     * @param newsFeed  the news feed
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)    
    public void notifyFeedAvailable (@Nonnull RSSFeed newsFeed);

    /*******************************************************************************************************************
     *
     * Notifies that there's no news feed available.
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void notifyFeedUnavailable();

    /*******************************************************************************************************************
     *
     * Notifies that all messages have been marked as read.
     * 
     * @param message   the notification message
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void notifyAllMessagesMarkedAsRead (@Nonnull String message);

    /*******************************************************************************************************************
     *
     * Asks for confirmation to download news.
     * 
     * @param questionWithFeedback   the question and the feedback
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.ANY)
    public void confirmToDownloadNews (@Nonnull QuestionWithFeedback questionWithFeedback);
  }
