/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet.commonnames;

import javax.inject.Provider;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.thread.ThreadType;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.role.ui.ThreadBoundLookupContext;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonomyPreferences;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TaxonFactSheetViewControllerSupport;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.presentationmodel.CommonNamesPresentationModel;
import static org.openide.util.NbBundle.*;

/***********************************************************************************************************************
 *
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultTaxonCommonNamesFactSheetViewController extends TaxonFactSheetViewControllerSupport<TaxonCommonNamesFactSheetView> 
                                                          implements TaxonCommonNamesFactSheetViewController
  {
    private static final Class<DefaultTaxonCommonNamesFactSheetViewController> _ = DefaultTaxonCommonNamesFactSheetViewController.class;
    
    private final Provider<TaxonomyPreferences> taxonomyPreferences = Locator.createProviderFor(TaxonomyPreferences.class);
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DefaultTaxonCommonNamesFactSheetViewController (final @Nonnull TaxonCommonNamesFactSheetView view,           
                                                           final @Nonnull Taxon taxon)
      {
        super(view, taxon);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    protected void createPresentationModels()
      {
        ThreadBoundLookupContext.with(this)
                      .run(new ThreadBoundLookupContext.Task<RuntimeException>() 
          {
            public void run() 
              {
                final List<CommonNamesPresentationModel> presentationModels = new ArrayList<CommonNamesPresentationModel>();

                for (final Locale locale : taxonomyPreferences.get().getTaxonomy().getLocales())
                  {
                    try
                      {
                        final String language = getMessage(_, "lang_" + locale.getLanguage());
                        final String name = taxon.getDisplayName(locale);
                        presentationModels.add(new CommonNamesPresentationModel(language, name, taxon.getScientificName()));
                      }
                    catch (NotFoundException e)
                      {
                        // ok, no name
                      }
                  }

                Collections.sort(presentationModels, new Comparator<CommonNamesPresentationModel>() 
                  {
                    public int compare (final @Nonnull CommonNamesPresentationModel o1, 
                                        final @Nonnull CommonNamesPresentationModel o2) 
                      {
                        return o1.getLanguage().compareTo(o2.getLanguage());
                      }
                  });

                for (final PresentationModel presentationModel : presentationModels) 
                  {
                    add(presentationModel);
                  }
              }
          });          
      }
  }
