/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet.main;

import javax.annotation.Nonnull;
import it.tidalwave.util.AsException;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.thread.ThreadType;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TaxonFactSheetView;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TaxonFactSheetViewController;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TaxonFactSheetViewControllerSupport;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.presentationmodel.TextPresentationModel;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.presentationmodel.DocumentPresentationModel;
import it.tidalwave.role.ui.ThreadBoundLookupContext;
import static it.tidalwave.role.HtmlRenderable.HtmlRenderable;
import static it.tidalwave.bluebill.mobile.taxonomy.ui.TaxonRenderingOptions.*;
import static it.tidalwave.bluebill.factsheet.Documentable.Documentable;
import static it.tidalwave.bluebill.factsheet.FactSheet.*;

/***********************************************************************************************************************
 *
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultTaxonMainFactSheetViewController extends TaxonFactSheetViewControllerSupport<TaxonFactSheetView>
                                                   implements TaxonFactSheetViewController<TaxonFactSheetView>
  {
    public DefaultTaxonMainFactSheetViewController (final @Nonnull TaxonFactSheetView view, final @Nonnull Taxon taxon)
      {
        super(view, taxon);
      }

    @ThreadConfined(type=ThreadType.NOT_UI)
    protected void createPresentationModels()
      {
        ThreadBoundLookupContext.with(this)
                      .run(new ThreadBoundLookupContext.Task<RuntimeException>() 
          {
            public void run() 
              {
                add(new TextPresentationModel(taxon.as(HtmlRenderable).render(HIERARCHY)));

                try
                  {
                    // FIXME: instead of tweaking h2, use a CSS
                    add(new DocumentPresentationModel(taxon.as(Documentable).getFactSheet().get(DESCRIPTION).replaceAll(("h2>"), "h3>")));
                  }
                catch (AsException e)
                  {
                    add(new TextPresentationModel("No info")); // FIXME
                  }
                catch (NotFoundException e)
                  {
                    add(new TextPresentationModel("No info")); // FIXME
                  }
              }
          });
      }
  }
