/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.role.ui;

import javax.annotation.Nonnull;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;
import org.openide.util.Lookup;

/***********************************************************************************************************************
 * 
 * A class which manages thread-scope injection of capabilities. It provides a method that binds a Task to a Lookup
 * and arranges things so that all PresentationModels created within the task can have extra capabilities injected from
 * the Lookup. FIXME: this should work with any AsLookupSuppor subclass, not only PresentationModel.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor(access= AccessLevel.PRIVATE)
public class ThreadBoundLookupContext 
  {
    public static interface Task<T extends Throwable>
      {
        public void run()
          throws T;
      }

    private final Lookup lookup;

    public static final ThreadLocal<Lookup> LOOKUP_THREAD_LOCAL = new ThreadLocal<Lookup>();
    
    @Nonnull
    public static ThreadBoundLookupContext with (final @Nonnull Lookup lookup)
      {
        return new ThreadBoundLookupContext(lookup);  
      }
    
    @Nonnull
    public static ThreadBoundLookupContext with (final @Nonnull Lookup.Provider lookupProvider)
      {
        return new ThreadBoundLookupContext(lookupProvider.getLookup());  
      }
    
    public <T extends Throwable> void run (final @Nonnull Task<T> task) 
      throws T
      {
        try
          {
            LOOKUP_THREAD_LOCAL.set(lookup);
            task.run();
          }
        finally
          {
            LOOKUP_THREAD_LOCAL.remove();
          }
      }
  }
