/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.role.ui.spi;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.role.ui.ThreadBoundLookupContext;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProvider;
import it.tidalwave.netbeans.util.AsLookupSupport;

/***********************************************************************************************************************
 *
 * TODO: should have an owner and include the owner Lookup.
 * As the NetBeans Node, it should allow children, have event listeners for children added/removed/changed.
 * This class so becomes the true M in MVC.
 * 
 * @stereotype Role
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultPresentationModel extends AsLookupSupport implements PresentationModel 
  {
    protected final PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
    
    @CheckForNull
    private transient Lookup threadLocalLookup;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DefaultPresentationModel()
      {        
        threadLocalLookup = ThreadBoundLookupContext.LOOKUP_THREAD_LOCAL.get();
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DefaultPresentationModel (final @Nonnull Object owner)
      {
        super(owner, new Object[]{ owner });
        threadLocalLookup = ThreadBoundLookupContext.LOOKUP_THREAD_LOCAL.get();
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DefaultPresentationModel (final @Nonnull Object[] instanceCapabilities)
      {
        super(instanceCapabilities);
        threadLocalLookup = ThreadBoundLookupContext.LOOKUP_THREAD_LOCAL.get();
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DefaultPresentationModel (final @Nonnull Object owner, final @Nonnull Object[] instanceCapabilities)
      {
        super(owner, instanceCapabilities);
        threadLocalLookup = ThreadBoundLookupContext.LOOKUP_THREAD_LOCAL.get();
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public synchronized void addPropertyChangeListener (final @Nonnull PropertyChangeListener listener)
      {
        propertyChangeSupport.addPropertyChangeListener(listener);
      }
        
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public synchronized void addPropertyChangeListener (final @Nonnull String propertyName,   
                                                        final @Nonnull PropertyChangeListener listener) 
      {
        propertyChangeSupport.addPropertyChangeListener(propertyName, listener);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public synchronized void removePropertyChangeListener (final @Nonnull PropertyChangeListener listener)
      {
        propertyChangeSupport.removePropertyChangeListener(listener);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public synchronized void removePropertyChangeListener (final @Nonnull String propertyName,
                                                           final @Nonnull PropertyChangeListener listener) 
      {
        propertyChangeSupport.removePropertyChangeListener(propertyName, listener);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public synchronized boolean hasListeners (final @Nonnull String propertyName) 
      {
        return propertyChangeSupport.hasListeners(propertyName);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    public synchronized PropertyChangeListener[] getPropertyChangeListeners()
      {
        return propertyChangeSupport.getPropertyChangeListeners();
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    public synchronized PropertyChangeListener[] getPropertyChangeListeners (final @Nonnull String propertyName)  
      {
        return propertyChangeSupport.getPropertyChangeListeners(propertyName);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected Lookup createLookup() 
      {
        if (threadLocalLookup == null)
          {
            return super.createLookup();  
          }
        
        final Lookup lookup = createThreadLocalLookup(this, super.createLookup());
        threadLocalLookup = null;
        
        return lookup;
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    // FIXME: push up to AsLookupSupport
    // FIXME: add a method to LookupFactory that specifies the Lookup instead of using Lookup.Provider and merge the code
    @Nonnull
    private Lookup createThreadLocalLookup (final @Nonnull Object owner,
                                           final @Nonnull Lookup ... extraLookups)
      {
//        log.trace("createThreadLocalLookup({}, ...)", owner);
        final List<Lookup> lookups = new ArrayList<Lookup>();
        
        for (final CapabilitiesProvider provider : threadLocalLookup.lookupAll(CapabilitiesProvider.class))
          {
            if (provider.getManagedClass().isAssignableFrom(owner.getClass()))
              {
                lookups.add(Lookups.fixed(provider.createCapabilities(owner).toArray()));
                lookups.addAll(provider.createLookups(owner));
              }
          }
          
        lookups.addAll(Arrays.asList(extraLookups));
//        log.trace(">>>> lookups for {}: {}", owner, extraLookups);
        
        return new ProxyLookup(lookups.toArray(new Lookup[lookups.size()]));
      }
  }
