/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.document;

import javax.annotation.Nonnull;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import it.tidalwave.mobile.io.IoUtils;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.logging.Logger;
import static  it.tidalwave.semantic.document.Document.*;

/***********************************************************************************************************************
 *
 * A {@link DocumentVisitor} which writes a {@link Document} and its attachments to a folder.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DocumentWriter extends DocumentVisitorSupport<IOException>
  {
    private static final String CLASS = DocumentWriter.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    @Nonnull
    private final String documentBaseName;
    
    @Nonnull
    private final File folder;
    
    public DocumentWriter (final @Nonnull String documentBaseName, final @Nonnull File folder) 
      {
        this.documentBaseName = documentBaseName;
        this.folder = folder;
      }
        
    // TODO: increment the ProgressListener
    @Override
    public void visitBody (final @Nonnull Document report) 
      throws IOException
      {
        write(new File(folder, documentBaseName + ".txt"), report);
      }

    @Override
    public void visitAttachment (final @Nonnull Document attachment) 
      throws IOException
      {
          {
            write(new File(folder, documentBaseName + getExtension(attachment)), attachment);
          }
      }
    
    @Nonnull
    private static String getExtension (final @Nonnull Document attachment) 
      {
        try 
          {
            final String ext = Document.EXTENSION_MAP_BY_MIME_TYPE.get(attachment.get(MIME_TYPE));
            return (ext == null) ? "" : "." + ext;
          } 
        catch (NotFoundException e)
          {
            return ""; // no mime type
          }
      }

    private void write (final @Nonnull File file, final @Nonnull Document report) 
      throws IOException
      {
        logger.info("write(%s, %s)", file.getAbsolutePath(), report);
        
        FileWriter w = null;

        try
          {
            w = new FileWriter(file);
            w.write(report.get(BODY));
          }
        catch (NotFoundException e)
          {
            // ok, no body  
          }
        finally 
          {
            IoUtils.safeClose(w); // TODO: use Lombok                    
          }                            
      }                        
  }
